/* 
 * Rearranja as linhas de uma matriz dada em ordem crescente, usando
 * os elementos de uma coluna especificada como as chaves de
 * ordenação.  Para colocar as linhas de uma matriz M por N em ordem
 * não-decrescente de acordo com os elementos da coluna k, faz-se
 * 
 * $ java-algs4 Generator 5 4 1 123 2023 
 *   2   1   3   1 
 *   3   3   1   3 
 *   2   1   3   3 
 *   1   2   2   1 
 *   2   1   3   1 
 * $ java-algs4 Generator 5 4 1 123 2023 | java-algs4 MergeRows 5 4 1
 *   2   1   3   1 
 *   2   1   3   3 
 *   2   1   3   1 
 *   1   2   2   1 
 *   3   3   1   3 
 * $ java-algs4 Generator 5 4 1 123 2023 | java-algs4 MergeRows 5 4 3
 *   2   1   3   1 
 *   1   2   2   1 
 *   2   1   3   1 
 *   3   3   1   3 
 *   2   1   3   3 
 * $ 
 */

import edu.princeton.cs.algs4.StdOut;
import edu.princeton.cs.algs4.StdIn;

public class MergeRows
{
    public static int[][] readIntMatrix(int M, int N) {
	int[][] a = new int[M][N];

	for (int i = 0; i < M; i++) 
	    for (int j = 0; j < N; j++)
		a[i][j] = StdIn.readInt();
	
	return a;
    }

    // merge a[lo..mid) and a[mid..hi) into a[lo..hi), supposing that
    // a[lo..mid) and a[mid..hi) are in non-decresing order
    public static void merge(int[][] a, int[][] aux, int lo, int mid, int hi, int p) {
        int i = lo, j = mid;
        for (int k = lo; k < hi; k++) {
            if      (i == mid)                 aux[k] = a[j++];
            else if (j == hi)                  aux[k] = a[i++];
            else if (a[j][p] - a[i][p] < 0)    aux[k] = a[j++];
            else                               aux[k] = a[i++];
        }

        // copy back
        for (int k = lo; k < hi; k++)
            a[k] = aux[k];
    }

    // sort a[lo..hi)
    public static void sort(int[][] a, int[][] aux, int lo, int hi, int p) {
        if (hi - lo <= 1) return;

        int mid = lo + (hi - lo) / 2;
        sort(a, aux, lo, mid, p);
        sort(a, aux, mid, hi, p);

        merge(a, aux, lo, mid, hi, p);
    }

    public static void sort(int[][] a, int p) {
        int n = a.length;
        int[][] aux = new int[n][];
        sort(a, aux, 0, n, p);
    }

    public static void show(int[][] a) {
	int M = a.length;
	int N = a[0].length;

	for (int i = 0; i < M; i++) {
	    for (int j = 0; j < N; j++)
		StdOut.printf("%2d ", a[i][j]);
	    StdOut.println();
	}
    }
    
    public static void main(String[] args)
    {
	int M = Integer.parseInt(args[0]);
	int N = Integer.parseInt(args[1]);
	int k = Integer.parseInt(args[2]);

	int[][] a = readIntMatrix(M, N);

	sort(a, k);

	show(a);
    }
}
